/**
 * read-file <https://github.com/assemble/read-file>
 *
 * Copyright (c) 2014 Jon Schlinkert, Brian Woodward, contributors.
 * Licensed under the MIT license.
 */
'use strict';

var fs = require('graceful-fs');
var async = require('async');
var normalize = require('file-normalize');
var file = module.exports = {};


// Read file synchronously
file.readFileSync = function(filepath, enc) {
  enc = enc || 'utf8';
  var buffer = fs.readFileSync(String(filepath), enc);
  try {
    return normalize.stripBOM(buffer);
  } catch (err) {
    err.message = 'Failed to read "' + filepath + '": ' + err.message;
    throw err;
  }
};

// Read file async
file.readFile = function (filepath, enc, callback) {
  if (enc && typeof enc === 'function') {
    callback = enc;
    enc = 'utf8';
  }

  async.waterfall([
    function (next) {
      fs.readFile(String(filepath),  (enc || 'utf8'), next);
    }, function (contents, next) {
      try {
        next(null, normalize.stripBOM(contents));
      } catch (err) {
        err.message = 'Failed to read "' + filepath + '": ' + err.message;
        next(err);
      }
    }
  ], callback);
};